/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2009 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id: Basic.java,v d7e8fed5e750 2009/09/17 23:55:58 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.metadata.basic;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.io.Serializable;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: Basic.java,v d7e8fed5e750 2009/09/17 23:55:58 fabrizio $
 *
 ******************************************************************************/
public class Basic implements Serializable
  {
    private static final long serialVersionUID = 3049504795930593059L;
    
    private static int idGenerator;
    
    private transient int id;

    @Nonnull
    private transient DataObject dataObject;

    private String mimeType;
    private boolean mimeTypeAvailable;
    private List<String> sidecars;
    private boolean sidecarsAvailable;
    private Long fileSize;
    private boolean fileSizeAvailable;
    private Date latestModificationDateTime;    
    private boolean latestModificationDateTimeAvailable;


    // For serialization
    public Basic()
      {
      }

    public Basic (@Nonnull final DataObject dataObject)
      {
        this.dataObject = dataObject;
      }

    @CheckForNull
    public String getMimeType() 
      {
        return mimeType;
      }

    public void setMimeType (final String mimeType) 
      {
        this.mimeType = mimeType;
      }

    public boolean isMimeTypeAvailable()
      {
        return mimeTypeAvailable;
      }

    public void setSidecars (final Collection<String> sideCars)
      {
        if (sideCars == null)
          {
            this.sidecars = null;  
          }
        
        else
          {
            if (this.sidecars == null)
              {
                this.sidecars = new ArrayList<String>();
              }
            
            this.sidecars.clear();
            this.sidecars.addAll(sideCars);
          }
      }
    
    @CheckForNull
    public Collection<String> getSidecars()
      {
        return (sidecars == null) ? null : Collections.unmodifiableCollection(sidecars);
      }
    
    public boolean isSidecarsAvailable()
      {
        return sidecarsAvailable;
      }

    public static Integer getIdGenerator() 
      {
        return idGenerator;
      }

    public static void setIdGenerator (final Integer idGenerator) 
      {
        Basic.idGenerator = idGenerator;
      }

    public void setSidecarsAvailable(boolean sidecarsAvailable) 
      {
        this.sidecarsAvailable = sidecarsAvailable;
      }

    public void setMimeTypeAvailable(boolean mimeTypeAvailable) 
      {
        this.mimeTypeAvailable = mimeTypeAvailable;
      } 

    public boolean isLatestModificationDateTimeAvailable()
      {
        return latestModificationDateTimeAvailable;
      }

    public void setLatestModificationDateTimeAvailable(boolean latestModificationDateTimeAvailable)
      {
        this.latestModificationDateTimeAvailable = latestModificationDateTimeAvailable;
      }

    @CheckForNull
    public Long getFileSize()
      {
        return fileSize;
      }

    public void setFileSize(Long fileSize)
      {
        this.fileSize = fileSize;
      }

    public boolean isFileSizeAvailable()
      {
        return fileSizeAvailable;
      }

    public void setFileSizeAvailable(boolean fileSizeAvailable)
      {
        this.fileSizeAvailable = fileSizeAvailable;
      }

    //////////////////////////////

    @Nonnull
    public String getFileName()
      {
        final FileObject primaryFile = (dataObject != null) ? dataObject.getPrimaryFile() : null;
        return (primaryFile != null) ? primaryFile.getExt() : null;
      }

    @Nonnull
    public String getFolder()
      {
        final FileObject primaryFile = (dataObject != null) ? dataObject.getPrimaryFile() : null;
        return (primaryFile != null) ? primaryFile.getParent().getNameExt() : null;
      }

    @Nonnull
    public String getMetadataStatus()
      {
        return "";
      }

    @CheckForNull
    public Date getLatestModificationDateTime()
      {
        return (latestModificationDateTime == null) ? null : new Date(latestModificationDateTime.getTime());
//        return (dataObject != null)
//               ? dataObject.getLookup().lookup(Metadata.class).getLatestModificationTime()
//               : null;
      }

    // FIXME: needed?
    public void setLatestModificationDateTime (Date latestModificationDateTime)
      {
        this.latestModificationDateTime = (latestModificationDateTime == null) ? null : new Date(latestModificationDateTime.getTime());
      }
    
    public boolean isAvailable()
      {
        return mimeTypeAvailable
          || sidecarsAvailable
          || latestModificationDateTimeAvailable    
          || fileSizeAvailable;
      }

    @Override
    public String toString()
      {
        synchronized (Basic.class)
          {
            if (id == 0)
              {
                id = ++idGenerator;
              }
          }
        
        return String.format("Basic@%d", id);
      }

    // Remember that id MUST not be a part of the comparison and you must use instanceof instead of getClass()
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof Basic)) {
            return false;
        }
        final Basic other = (Basic) obj;
        if (this.mimeType != other.mimeType && (this.mimeType == null || !this.mimeType.equals(other.mimeType))) {
            return false;
        }
        if (this.mimeTypeAvailable != other.mimeTypeAvailable) {
            return false;
        }
        if (this.sidecars != other.sidecars && (this.sidecars == null || !this.sidecars.equals(other.sidecars))) {
            return false;
        }
        if (this.sidecarsAvailable != other.sidecarsAvailable) {
            return false;
        }
        if (this.latestModificationDateTime != other.latestModificationDateTime && (this.latestModificationDateTime == null || !this.latestModificationDateTime.equals(other.latestModificationDateTime))) {
            return false;
        }
        if (this.latestModificationDateTimeAvailable != other.latestModificationDateTimeAvailable) {
            return false;
        }

        if (this.fileSize != other.fileSize && (this.fileSize == null || !this.fileSize.equals(other.fileSize))) {
            return false;
        }
        if (this.fileSizeAvailable != other.fileSizeAvailable) {
            return false;
        }
        return true;
    }

    // Remember that id MUST not be a part of the comparison
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 43 * hash + (this.mimeType != null ? this.mimeType.hashCode() : 0);
        hash = 43 * hash + (this.mimeTypeAvailable ? 1 : 0);
        hash = 43 * hash + (this.sidecars != null ? this.sidecars.hashCode() : 0);
        hash = 43 * hash + (this.sidecarsAvailable ? 1 : 0);
        hash = 43 * hash + (this.latestModificationDateTime != null ? this.latestModificationDateTime.hashCode() : 0);
        hash = 43 * hash + (this.latestModificationDateTimeAvailable ? 1 : 0);
        hash = 43 * hash + (this.fileSize != null ? this.fileSize.hashCode() : 0);
        hash = 43 * hash + (this.fileSizeAvailable ? 1 : 0);

        return hash;
      }
  }  
